"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const file_temp_1 = require("../../libs/file-temp");
const testing_1 = require("../../testing");
const yaml_1 = require("yaml");
const scene_1 = require("./scene");
let scene;
let encryptedFile;
const password = 'example';
beforeEach(async () => {
    testing_1.Testing.reset();
    scene = testing_1.Testing.newElement(scene_1.Scene);
});
afterEach(async () => {
    await scene.dispose();
});
afterAll(() => {
    encryptedFile && (0, fs_1.unlinkSync)(encryptedFile);
});
test('Should load vars from yaml file', async () => {
    await scene.dispose();
    const tmpFile = new file_temp_1.FileTemp('.yaml');
    try {
        await tmpFile.create((0, yaml_1.stringify)({
            name: 'name 1',
            age: 1,
            male: true,
            more: {
                address: '1',
                num: 2,
                married: false
            }
        }));
        scene = testing_1.Testing.newElement(scene_1.Scene, true);
        await scene.init({
            content: `
vars_file: ${tmpFile.file}
runs:
  - Test env
`
        });
        await scene.exec();
        expect(scene.sceneVars.name).toBe('name 1');
        expect(scene.sceneVars.age).toBe(1);
        expect(scene.sceneVars.male).toBe(true);
        expect(scene.sceneVars.more).toEqual({
            address: '1',
            num: 2,
            married: false
        });
    }
    finally {
        tmpFile.remove();
    }
});
test('Should load vars from json file', async () => {
    await scene.dispose();
    const tmpFile = new file_temp_1.FileTemp('.json');
    try {
        await tmpFile.create(JSON.stringify({
            name: 'name 1',
            age: 1,
            male: true,
            more: {
                address: '1',
                num: 2,
                married: false
            }
        }));
        scene = testing_1.Testing.newElement(scene_1.Scene, true);
        await scene.init({
            content: `
vars_file: ${tmpFile.file}
runs:
  - Test env
`
        });
        await scene.exec();
        expect(scene.sceneVars.name).toBe('name 1');
        expect(scene.sceneVars.age).toBe(1);
        expect(scene.sceneVars.male).toBe(true);
        expect(scene.sceneVars.more).toEqual({
            address: '1',
            num: 2,
            married: false
        });
    }
    finally {
        tmpFile.remove();
    }
});
test('Should override env when load scene', async () => {
    await scene.dispose();
    process.env.NAME = 'name 1';
    process.env.age = '1';
    process.env.male = '1';
    process.env.more_address = '1';
    process.env.MORE_NUM = '2';
    process.env.MORE_MARRied = 'no';
    scene = testing_1.Testing.newElement(scene_1.Scene, true);
    await scene.init({
        content: `
vars:
  name: name 0
  age: 0
  male: false
  more:
    address: addr 1
    num: 1
    married: true
runs:
  - Test env
`
    });
    await scene.exec();
    expect(scene.sceneVars.name).toBe('name 1');
    expect(scene.sceneVars.age).toBe(1);
    expect(scene.sceneVars.male).toBe(true);
    expect(scene.sceneVars.more).toEqual({
        address: '1',
        num: 2,
        married: false
    });
});
test('Should load successfully from text content with only list items', async () => {
    await scene.init({ content: `- echo: Hello` });
    await scene.exec();
    expect(scene.result).toHaveLength(1);
});
test('Should load successfully from text content with full props', async () => {
    await scene.init({
        content: `
title: test
runs: 
  - echo: Hello
`
    });
    await scene.exec();
    expect(scene.result).toHaveLength(1);
});
test('Should load successfully from file', async () => {
    const tmp = new file_temp_1.FileTemp();
    try {
        await tmp.create(`
title: Test here
runs:
  - Hello
  - World
`);
        await scene.init({ path: tmp.file });
        await scene.exec();
        expect(scene.result).toHaveLength(2);
    }
    finally {
        tmp.remove();
    }
});
test('Should generate a generated scene file when set password', async () => {
    const tmp = new file_temp_1.FileTemp('.yaml');
    try {
        await tmp.create(`
title: Test here
password: ${password}
runs:
  - Hello
  - World
`);
        await scene.init({ path: tmp.file });
        await scene.exec();
        expect(scene.encryptedPath && (0, fs_1.existsSync)(scene.encryptedPath)).toBe(true);
        encryptedFile = scene.encryptedPath || '';
    }
    finally {
        tmp.remove();
    }
});
test('Load a encrypted scene file', async () => {
    await scene.init({ path: encryptedFile, password });
    await scene.exec();
    expect(scene.result).toHaveLength(2);
});
test('Should run echo element', async () => {
    await scene.init({
        content: `
- echo:
    content: 'ok'
`
    });
    const [echo] = await scene.exec();
    expect(echo.result).toBe('ok');
    await scene.dispose();
});
//# sourceMappingURL=scene.spec.js.map